#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


from pyramid.decorator import reify
from pyramid.exceptions import NotFound

from pyams_default_theme.page import BaseIndexPage
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.interfaces import IPortalContext, IPortalPage, IPortalPortletsConfiguration, \
    IPortalTemplateConfiguration, IPortlet, IPortletCSSClass, IPortletRenderer, PREVIEW_MODE
from pyams_skin.layer import IPyAMSUserLayer
from pyams_template.template import template_config
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
from pyams_workflow.interfaces import IWorkflowPublicationInfo


class BasePortalContextIndexPage(BaseIndexPage):
    """Base portal context index page"""

    portlets = None

    def update(self):
        super(BasePortalContextIndexPage, self).update()
        # extract all renderers list
        self.portlets = {}
        template_configuration = self.template_configuration
        portlets_configuration = self.portlet_configuration
        for row_id in range(template_configuration.rows):
            for slot_name in template_configuration.get_slots(row_id):
                for portlet_id in template_configuration.slot_config[slot_name].portlet_ids:
                    settings = portlets_configuration.get_portlet_configuration(portlet_id).settings
                    renderer = self.request.registry.queryMultiAdapter((self.context, self.request, self, settings),
                                                                       IPortletRenderer,
                                                                       name=settings.renderer)
                    if renderer is not None:
                        renderer.update()
                        self.portlets[portlet_id] = renderer

    @reify
    def page(self):
        return IPortalPage(self.context)

    @reify
    def template_configuration(self):
        return IPortalTemplateConfiguration(self.page.template)

    @reify
    def portlet_configuration(self):
        return IPortalPortletsConfiguration(self.context)

    def get_portlet(self, name):
        return self.request.registry.queryUtility(IPortlet, name=name)

    def get_portlet_css_class(self, portlet_id):
        configuration = self.portlet_configuration.get_portlet_configuration(portlet_id)
        portlet = self.get_portlet(configuration.portlet_name)
        if portlet is not None:
            return self.request.registry.queryMultiAdapter((portlet, self.request), IPortletCSSClass, default='')

    def render_portlet(self, portlet_id, template_name=''):
        renderer = self.portlets.get(portlet_id)
        if renderer is not None:
            return renderer.render(template_name)
        return ''


@pagelet_config(name='', context=IPortalContext, layer=IPyAMSUserLayer)
@template_config(template='templates/pagelet.pt', layer=IPyAMSUserLayer)
class PortalContextIndexPage(BasePortalContextIndexPage):
    """Portal context index page"""

    def update(self):
        wf_info = IWorkflowPublicationInfo(self.context, None)
        if (wf_info is not None) and not wf_info.is_visible(self.request):
            raise NotFound()
        super(PortalContextIndexPage, self).update()


@pagelet_config(name='preview.html', context=IPortalContext, layer=IPyAMSUserLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class PortalContextPreviewPage(PortalContextIndexPage):
    """Portal context preview page"""

    def update(self):
        # Bypass publication status in preview
        self.request.annotations[PREVIEW_MODE] = True
        super(PortalContextIndexPage, self).update()
